
clear all; close all; clc

g = 9.81;
uhover = 8.96; %Thrust value needed for hover (empirically measured); see Kene's email (flagged)
umin = 0; umax = 10; %bounds on input in 'counts'.

kT = g/(uhover^2);
kp = 1/0.15;

xeq = [2 0 sqrt(g/kT)]';
ueq = sqrt(g/kT); %i.e. the same as uhover.

velMin = -1.5; velMax = 1.5; %bounds on velocity in m/s.

% Visualize vector field about hover (in the last two dimensions)
u = linspace(umin,umax,6);
for i = 1:6
    subplot(2,3,i), hold on
    x2 = linspace(velMin,velMax,15); 
    x3 = linspace(umin,umax,15); %(assuming no input error: x3=u)
    [X2,X3] = meshgrid(x2, x3);
    
    %Nonlinear vector field
    quiver(X2,X3, kT*X3.^2-g, kp*(u(i)-X3))
    
    %Linear approximation:
    quiver(X2,X3, 2*kT*xeq(3)*(X3-xeq(3)), -kp*(X3-xeq(3))+kp*(u(i)-ueq),'g')
    
    plot(0,sqrt(g/kT),'ro','MarkerSize',6, 'MarkerFaceColor','r')
    xlabel('x_2'), ylabel('x_3'), title(['u = ' num2str(u(i))])
    axis([velMin velMax umin umax])
end


%Error vector field
figure,
for i = 1:6
    subplot(2,3,i), hold on
    D2 = (kT*X3.^2-g) - (2*kT*xeq(3)*(X3-xeq(3)));
    D3 = (kp*(u(i)-X3)) - (-kp*(X3-xeq(3))+kp*(u(i)-ueq));
    quiver(X2,X3, D2,D3,'r')
    plot(0,sqrt(g/kT),'ro','MarkerSize',6, 'MarkerFaceColor','r')
    xlabel('x_2'), ylabel('x_3'), title(['v for u = ' num2str(u(i))])
    axis([velMin velMax umin umax])
end


%Disturbance
figure,
for i = 1:6
    subplot(2,3,i), hold on
    D2 = ( kT*X3.^2-g ) - ( 2*kT*xeq(3)*(X3-xeq(3)) );
    D3 = ( kp*(u(i)-X3) ) - ( -kp*(X3-xeq(3))+kp*(u(i)-ueq) );
    surf(X2,X3, sqrt(D2.^2+D3.^2))
    plot(0,sqrt(g/kT),'ro','MarkerSize',6, 'MarkerFaceColor','r')
    xlabel('x_2'), ylabel('x_3'), zlabel('||v||'), title(['u = ' num2str(u(i))])
    view(-53,30), grid
    xlim([velMin velMax])
end


figure, hold on
i = 1;
D2 = ( kT*X3.^2-g ) - ( 2*kT*xeq(3)*(X3-xeq(3)) );
D3 = ( kp*(u(i)-X3) ) - ( -kp*(X3-xeq(3))+kp*(u(i)-ueq) );
surf(X2,X3, sqrt(D2.^2+D3.^2))
plot3(0,sqrt(g/kT), 0.02, 'ro','MarkerSize',6, 'MarkerFaceColor','r')
xlabel('$x_2$','Interpreter','Latex'), ylabel('$x_3$','Interpreter','Latex'), zlabel('$||v||$','Interpreter','Latex')
set(get(gca,'xlabel'),'fontsize',20)
set(get(gca,'ylabel'),'fontsize',20)
set(get(gca,'zlabel'),'fontsize',20)
grid
xlim([velMin velMax])
view([-104 18])
%colormap cool








% figure, 
% 
% d2 = (kT*x3.^2-g) - (2*kT*xeq(3)*(x3-xeq(3)));
% d3 = (kp*(u(i)-x3)) - (-kp*(x3-xeq(3))+kp*(u(i)-ueq));
% 
% for i = 1:6
%     subplot(2,3,i), hold on
%     plot(x2, d2);
%     %vline(0);
%     xlabel('x_2'), ylabel('d_2'), title(['u = ' num2str(u(i))])
% end
% 
% figure, 
% 
% for i = 1:6
%     subplot(2,3,i), hold on
%     plot(x3, d3);
%     %vline(sqrt(g/kT));
%     xlabel('x_3'), ylabel('d_3'), title(['u = ' num2str(u(i))])
% end




% u = uhover;
% x1 = linspace(0,3,10);
% x2 = linspace(-4,4,10);
% x3 = linspace(umin,umax,10);
% [X1,X2,X3] = meshgrid(x1, x2, x3);
% quiver3(X1,X2,X3, X2,kT*X3.^2-g,kp*(u-X3))
% xlabel('x_1'), ylabel('x_2'), zlabel('x_3')


% A0 = [0 1 0;
%     0 0 2*kT*xeq(3);
%     0 0 -kp];




